package exemplos;

import java.awt.*;
import java.awt.event.*;
import java.awt.image.BufferedImage;
import java.io.IOException;
import java.net.*;
import javax.imageio.ImageIO;
import javax.swing.*;

public class MostraMultiplasImagens extends JFrame implements ActionListener
  {
  private JDesktopPane desktop;
  private JTextField url;
  private JComboBox escala;
  private String[] escalas = {"0.01","0.05","0.1","0.2","0.5","1","2","5","10","20"};
  
  public MostraMultiplasImagens()
    {
    desktop = new JDesktopPane(); 
    JPanel controle = new JPanel(new FlowLayout(FlowLayout.LEFT));
    controle.add(new JLabel("URL da Imagem:"));
    url = new JTextField(50);
    url.addActionListener(this);
    url.setText("http://www.lac.inpe.br/~rafael.santos/JIPCookbook/"+
                "MiscResources/Datasets/Landsat7_218076_742.png");
    controle.add(url);
    controle.add(new JLabel("Escala:"));
    escala = new JComboBox(escalas);
    escala.setSelectedIndex(5); 
    controle.add(escala);
    getContentPane().add(controle,BorderLayout.NORTH);
    getContentPane().add(desktop,BorderLayout.CENTER);
    setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    setSize(800,600); 
    setVisible(true); 
    }

  public void actionPerformed(ActionEvent e)
    {
    BufferedImage imagem = null;
    boolean carregada = true;
    try
      {
      imagem = ImageIO.read(new URL(url.getText()));
      }
    catch (MalformedURLException e1)
      {
      JOptionPane.showMessageDialog(this,"Erro na URL: "+url.getText(),
                                         "Erro na URL",JOptionPane.ERROR_MESSAGE);
      carregada = false;
      } 
    catch (IOException e1)
      {
      JOptionPane.showMessageDialog(this,"Erro de IO: "+e1.getMessage(),
                                         "Erro de IO",JOptionPane.ERROR_MESSAGE);
      carregada = false;
      }
    if (carregada)
      {
      if (imagem == null)
        {
        JOptionPane.showMessageDialog(this,"Não pode ler "+url.getText(),
                                           "Não pode ler",JOptionPane.ERROR_MESSAGE);
        }
      else 
        {
        float usaEscala = Float.parseFloat(escalas[escala.getSelectedIndex()]);
        ImagemIF i = new ImagemIF(url.getText(),usaEscala,new ImageIcon(imagem));
        desktop.add(i);
        }
      }
    }
   
  public static void main(String[] args)
    {
    new MostraMultiplasImagens();
    }
  
  }
