package simpleinvaders0;

import java.awt.*;
import java.awt.event.*;

import javax.swing.JPanel;

// Classe que implementa o painel do jogo. O painel controlará os elementos
// do jogo e a renderização.
public class InvadersPanel extends JPanel implements Runnable,KeyListener
  {
  // Dimensões da área do jogo.
  private static final int largura = 800;
  private static final int altura = 600;
  
  // Uma thread para controlar a animação.
  private Thread animator;         
  private boolean isPaused = false;

  // Teremos alguns UFOs passeando na tela.
  private Invader[] invasores;
  // O shooter e sua direção de movimento.
  private Shooter shooter;
  private Direcao dir;
  
  // Construtor, inicializa estruturas, registra interfaces, etc.  
  public InvadersPanel()
    {
    setBackground(Color.WHITE);
    setPreferredSize(new Dimension(largura,altura));
    setFocusable(true);
    requestFocus(); 
    addKeyListener(this);
    invasores = new Invader[20];
    for(int i=0;i<invasores.length;i++) invasores[i] = new Invader(this.getPreferredSize());
    shooter = new Shooter(this.getPreferredSize());
    }
  
  // Avisa que agora temos a interface em um container parente. 
  public void addNotify()
    { 
    super.addNotify();  
    startGame();        
    }

  // Iniciamos o jogo (e a thread de controle)
  private void startGame()
    { 
    if (animator == null) 
      {
      animator = new Thread(this);
      animator.start();
      }
    } 

  // Laço principal do jogo.  
  public void run()
    {
    while(true) 
      {
      gameUpdate();
      repaint();
      try
        {
        Thread.sleep(10);
        }
      catch (InterruptedException e)
        {
        e.printStackTrace();
        }
      }
    }

  // Atualizamos os elementos do jogo.
  private void gameUpdate()
    {
    if (!isPaused)
      {
      for(Invader i:invasores) i.move();
      shooter.move(dir);
      }
    }
  
  // Desenhamos o componente (e seus elementos)
  protected void paintComponent(Graphics g)
    {
    super.paintComponent(g);
    g.setColor(Color.BLACK);
    g.drawRect(0,0,getWidth(),getHeight()-100);
    for(Invader i:invasores) i.draw(g);
    shooter.draw(g);
    }

  // Processamos teclas pressionadas durante o jogo.
  public void keyPressed(KeyEvent e)
    {
    int keyCode = e.getKeyCode();
    if (keyCode == KeyEvent.VK_P) isPaused = !isPaused;
    if (isPaused) return;
    if (keyCode == KeyEvent.VK_LEFT) dir = Direcao.LEFT;
    else if (keyCode == KeyEvent.VK_RIGHT) dir = Direcao.RIGHT;
    else if (keyCode == KeyEvent.VK_UP) dir = Direcao.UP;
    else if (keyCode == KeyEvent.VK_DOWN) dir = Direcao.DOWN;
    }

  // Estes métodos servem para satisfazer a interface KeyListener
  public void keyReleased(KeyEvent e) { }

  public void keyTyped(KeyEvent e) { }
 
  
  }
